@@ -148,40 +148,6 @@ getprogname (void)
 
 #endif /* HAVE_GETPROGNAME */
 
-char *
-p11_basename (const char *name)
-{
-#ifdef OS_WIN32
-	static const char *delims = "/\\";
-#else
-	static const char *delims = "/";
-#endif
-
-	const char *end;
-	const char *beg;
-
-	if (name == NULL)
-		return NULL;
-
-	/* Any trailing slashes */
-	end = name + strlen (name);
-	while (end != name) {
-		if (!strchr (delims, *(end - 1)))
-			break;
-		end--;
-	}
-
-	/* Find the last slash after those */
-	beg = end;
-	while (beg != name) {
-		if (strchr (delims, *(beg - 1)))
-			break;
-		beg--;
-	}
-
-	return strndup (beg, end - beg);
-}
-
 #ifdef OS_UNIX
 #include <sys/stat.h>
 #include <sys/mman.h>
@@ -226,7 +192,7 @@ p11_mmap_open (const char *path,
 	if (map == NULL)
 		return NULL;
 
-	map->fd = open (path, O_RDONLY);
+	map->fd = open (path, O_RDONLY | O_CLOEXEC);
 	if (map->fd == -1) {
 		free (map);
 		return NULL;
@@ -326,14 +292,20 @@ p11_mmap_open (const char *path,
 	p11_mmap *map;
 
 	map = calloc (1, sizeof (p11_mmap));
-	if (map == NULL)
+	if (map == NULL) {
+		errno = ENOMEM;
 		return NULL;
+	}
 
 	map->file  = CreateFile (path, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_FLAG_RANDOM_ACCESS, NULL);
 	if (map->file == INVALID_HANDLE_VALUE) {
 		errn = GetLastError ();
 		free (map);
 		SetLastError (errn);
+		if (errn == ERROR_PATH_NOT_FOUND || errn == ERROR_FILE_NOT_FOUND)
+			errno = ENOENT;
+		else if (errn == ERROR_ACCESS_DENIED)
+			errno = EPERM;
 		return NULL;
 	}
 
@@ -342,6 +314,8 @@ p11_mmap_open (const char *path,
 		CloseHandle (map->file);
 		free (map);
 		SetLastError (errn);
+		if (errn == ERROR_ACCESS_DENIED)
+			errno = EPERM;
 		return NULL;
 	}
 
@@ -351,6 +325,8 @@ p11_mmap_open (const char *path,
 		CloseHandle (map->file);
 		free (map);
 		SetLastError (errn);
+		if (errn == ERROR_ACCESS_DENIED)
+			errno = EPERM;
 		return NULL;
 	}
 
@@ -362,6 +338,8 @@ p11_mmap_open (const char *path,
 		CloseHandle (map->file);
 		free (map);
 		SetLastError (errn);
+		if (errn == ERROR_ACCESS_DENIED)
+			errno = EPERM;
 		return NULL;
 	}
 
@@ -704,7 +682,7 @@ _gettemp (char *path,
 
 	for (;;) {
 		if (doopen) {
-			if ((*doopen = open (path, O_BINARY | O_CREAT | O_EXCL | O_RDWR, 0600)) >= 0)
+			if ((*doopen = open (path, O_BINARY | O_CREAT | O_EXCL | O_RDWR | O_CLOEXEC, 0600)) >= 0)
 				return (1);
 			if (errno != EEXIST)
 				return (0);
@@ -775,3 +753,47 @@ mkdtemp (char *template)
 }
 
 #endif /* HAVE_MKDTEMP */
+
+#ifndef HAVE_GETAUXVAL
+
+unsigned long
+getauxval (unsigned long type)
+{
+	static unsigned long secure = 0UL;
+	static bool check_secure_initialized = false;
+
+	/*
+	 * This is the only one our stand-in impl supports and is
+	 * also the only type we define in compat.h header
+	 */
+	assert (type == AT_SECURE);
+
+	if (!check_secure_initialized) {
+#if defined(HAVE_ISSETUGID)
+		secure = issetugid ();
+
+#elif defined(OS_UNIX)
+		uid_t ruid, euid, suid; /* Real, effective and saved user ID's */
+		gid_t rgid, egid, sgid; /* Real, effective and saved group ID's */
+
+#ifdef HAVE_GETRESUID
+		if (getresuid (&ruid, &euid, &suid) != 0 ||
+		    getresgid (&rgid, &egid, &sgid) != 0)
+#endif /* HAVE_GETRESUID */
+		{
+			suid = ruid = getuid ();
+			sgid = rgid = getgid ();
+			euid = geteuid ();
+			egid = getegid ();
+		}
+
+		secure = (ruid != euid || ruid != suid ||
+		          rgid != egid || rgid != sgid);
+#endif /* OS_UNIX */
+		check_secure_initialized = true;
+	}
+
+	return secure;
+}
+
+#endif /* HAVE_GETAUXVAL */
