@@ -68,6 +68,10 @@
 #define O_BINARY 0
 #endif
 
+#ifndef O_CLOEXEC
+#define O_CLOEXEC 0
+#endif
+
 #ifndef HAVE_GETPROGNAME
 const char * getprogname (void);
 #endif
@@ -84,6 +88,8 @@ char *       mkdtemp     (char *template);
 
 #endif /* HAVE_MKDTEMP */
 
+char *       strdup_path_mangle (const char *template);
+
 /* -----------------------------------------------------------------------------
  * WIN32
  */
@@ -214,13 +220,6 @@ void        p11_mmap_close  (p11_mmap *map);
 
 #endif /* OS_UNIX */
 
-/*
- * The semantics of both POSIX basename() and GNU asename() are so crappy that
- * we just don't even bother. And what's worse is how it completely changes
- * behavior if _GNU_SOURCE is defined. Nasty stuff.
- */
-char *       p11_basename   (const char *name);
-
 /* ----------------------------------------------------------------------------
  * MORE COMPAT
  */
@@ -296,4 +295,16 @@ time_t      timegm          (struct tm *tm);
 
 #endif /* HAVE_TIMEGM */
 
+#ifdef HAVE_GETAUXVAL
+
+#include <sys/auxv.h>
+
+#else /* !HAVE_GETAUXVAL */
+
+unsigned long     getauxval (unsigned long type);
+
+#define AT_SECURE 23
+
+#endif /* !HAVE_GETAUXVAL */
+
 #endif /* __COMPAT_H__ */
