@@ -44,7 +44,7 @@
 #include "debug.h"
 #include "hash.h"
 #include "index.h"
-#include "library.h"
+#include "message.h"
 #include "oid.h"
 #include "pkcs11x.h"
 #include "x509.h"
@@ -110,7 +110,6 @@ lookup_extension (p11_builder *builder,
 	CK_OBJECT_CLASS klass = CKO_X_CERTIFICATE_EXTENSION;
 	CK_OBJECT_HANDLE obj;
 	CK_ATTRIBUTE *attrs;
-	unsigned char *ext;
 	void *value;
 	size_t length;
 	node_asn *node;
@@ -122,6 +121,8 @@ lookup_extension (p11_builder *builder,
 		{ CKA_INVALID },
 	};
 
+	assert (ext_len != NULL);
+
 	/* Look for a stapled certificate extension */
 	match[0].pValue = p11_attrs_find_value (cert, CKA_ID, &length);
 	if (match[0].pValue != NULL) {
@@ -132,9 +133,10 @@ lookup_extension (p11_builder *builder,
 		if (attrs != NULL) {
 			value = p11_attrs_find_value (attrs, CKA_VALUE, ext_len);
 			if (value != NULL) {
-				ext = memdup (value, *ext_len);
-				return_val_if_fail (ext != NULL, NULL);
-				return ext;
+				if (*ext_len)
+					value = memdup (value, *ext_len);
+				return_val_if_fail (value != NULL, NULL);
+				return value;
 			}
 		}
 	}
@@ -227,16 +229,72 @@ calc_check_value (const unsigned char *data,
 	memcpy (check_value, checksum, 3);
 }
 
+static int
+atoin (const char *p,
+       int digits)
+{
+	int ret = 0, base = 1;
+	while(--digits >= 0) {
+		if (p[digits] < '0' || p[digits] > '9')
+			return -1;
+		ret += (p[digits] - '0') * base;
+		base *= 10;
+	}
+	return ret;
+}
+
+static int
+century_for_two_digit_year (int year)
+{
+	time_t now;
+	struct tm tm;
+	int century, current;
+
+	return_val_if_fail (year >= 0 && year <= 99, -1);
+
+	/* Get the current year */
+	now = time (NULL);
+	return_val_if_fail (now >= 0, -1);
+	if (!gmtime_r (&now, &tm))
+		return_val_if_reached (-1);
+
+	current = (tm.tm_year % 100);
+	century = (tm.tm_year + 1900) - current;
+
+	/*
+	 * Check if it's within 40 years before the
+	 * current date.
+	 */
+	if (current < 40) {
+		if (year < current)
+			return century;
+		if (year > 100 - (40 - current))
+			return century - 100;
+	} else {
+		if (year < current && year > (current - 40))
+			return century;
+	}
+
+	/*
+	 * If it's after then adjust for overflows to
+	 * the next century.
+	 */
+	if (year < current)
+		return century + 100;
+	else
+		return century;
+}
+
 static bool
 calc_date (node_asn *node,
            const char *field,
            CK_DATE *date)
 {
 	node_asn *choice;
-	struct tm when;
 	char buf[64];
-	time_t timet;
+	int century;
 	char *sub;
+	int year;
 	int len;
 	int ret;
 
@@ -252,39 +310,43 @@ calc_date (node_asn *node,
 
 	sub = strconcat (field, ".", buf, NULL);
 
+	/*
+	 * So here we take a shortcut and just copy the date from the
+	 * certificate into the CK_DATE. This doesn't take into account
+	 * time zones. However the PKCS#11 spec does not say what timezone
+	 * the dates are in. In the PKCS#11 value have a day resolution,
+	 * and time zones aren't that critical.
+	 */
+
 	if (strcmp (buf, "generalTime") == 0) {
 		len = sizeof (buf) - 1;
 		ret = asn1_read_value (node, sub, buf, &len);
 		return_val_if_fail (ret == ASN1_SUCCESS, false);
-		timet = p11_asn1_parse_general (buf, &when);
+		return_val_if_fail (len >= 8, false);
+
+		/* Same as first 8 characters of date */
+		memcpy (date, buf, 8);
 
 	} else if (strcmp (buf, "utcTime") == 0) {
 		len = sizeof (buf) - 1;
 		ret = asn1_read_value (node, sub, buf, &len);
 		return_val_if_fail (ret == ASN1_SUCCESS, false);
-		timet = p11_asn1_parse_utc (buf, &when);
+		return_val_if_fail (len >= 6, false);
+
+		year = atoin (buf, 2);
+		return_val_if_fail (year >= 0, false);
+
+		century = century_for_two_digit_year (year);
+		return_val_if_fail (century >= 0, false);
+
+		snprintf ((char *)date->year, 3, "%02d", century);
+		memcpy (((char *)date) + 2, buf, 6);
 
 	} else {
 		return_val_if_reached (false);
 	}
 
 	free (sub);
-
-	if (timet < 0)
-		return false;
-
-	assert (sizeof (date->year) == 4);
-	snprintf ((char *)buf, 5, "%04d", 1900 + when.tm_year);
-	memcpy (date->year, buf, 4);
-
-	assert (sizeof (date->month) == 2);
-	snprintf ((char *)buf, 3, "%02d", when.tm_mon + 1);
-	memcpy (date->month, buf, 2);
-
-	assert (sizeof (date->day) == 2);
-	snprintf ((char *)buf, 3, "%02d", when.tm_mday);
-	memcpy (date->day, buf, 2);
-
 	return true;
 }
 
@@ -630,7 +692,7 @@ const static builder_schema assertion_schema = {
 	GENERATED_CLASS,
 	{ COMMON_ATTRS,
 	  { CKA_X_PURPOSE, REQUIRE | CREATE },
-	  { CKA_VALUE, CREATE },
+	  { CKA_X_CERTIFICATE_VALUE, CREATE },
 	  { CKA_X_ASSERTION_TYPE, REQUIRE | CREATE },
 	  { CKA_ISSUER, CREATE },
 	  { CKA_SERIAL_NUMBER, CREATE },
@@ -1181,6 +1243,7 @@ build_assertions (p11_array *array,
 	CK_ATTRIBUTE autogen = { CKA_X_GENERATED, &truev, sizeof (truev) };
 	CK_ATTRIBUTE purpose = { CKA_X_PURPOSE, };
 	CK_ATTRIBUTE invalid = { CKA_INVALID, };
+	CK_ATTRIBUTE certificate_value = { CKA_X_CERTIFICATE_VALUE, };
 
 	CK_ATTRIBUTE *issuer;
 	CK_ATTRIBUTE *serial;
@@ -1191,7 +1254,7 @@ build_assertions (p11_array *array,
 	int i;
 
 	if (type == CKT_X_DISTRUSTED_CERTIFICATE) {
-		value = &invalid;
+		certificate_value.type = CKA_INVALID;
 		issuer = p11_attrs_find_valid (cert, CKA_ISSUER);
 		serial = p11_attrs_find_valid (cert, CKA_SERIAL_NUMBER);
 
@@ -1209,6 +1272,9 @@ build_assertions (p11_array *array,
 			p11_debug ("not building positive trust assertion for certificate without value");
 			return;
 		}
+
+		certificate_value.pValue = value->pValue;
+		certificate_value.ulValueLen = value->ulValueLen;
 	}
 
 	label = p11_attrs_find (cert, CKA_LABEL);
@@ -1224,7 +1290,7 @@ build_assertions (p11_array *array,
 
 		attrs = p11_attrs_build (NULL, &klass, &private, &modifiable,
 		                         id, label, &assertion_type, &purpose,
-		                         issuer, serial, value, &autogen, NULL);
+		                         issuer, serial, &certificate_value, &autogen, NULL);
 		return_if_fail (attrs != NULL);
 
 		if (!p11_array_push (array, attrs))
@@ -1304,7 +1370,7 @@ replace_trust_assertions (p11_builder *builder,
 	CK_RV rv;
 
 	CK_ATTRIBUTE match_positive[] = {
-		{ CKA_VALUE, },
+		{ CKA_X_CERTIFICATE_VALUE, },
 		{ CKA_CLASS, &assertion, sizeof (assertion) },
 		{ CKA_X_GENERATED, &generated, sizeof (generated) },
 		{ CKA_INVALID }
@@ -1321,7 +1387,8 @@ replace_trust_assertions (p11_builder *builder,
 	value = p11_attrs_find_valid (cert, CKA_VALUE);
 	if (value) {
 		positives = p11_array_new (NULL);
-		memcpy (match_positive, value, sizeof (CK_ATTRIBUTE));
+		match_positive[0].pValue = value->pValue;
+		match_positive[0].ulValueLen = value->ulValueLen;
 	}
 
 	issuer = p11_attrs_find_valid (cert, CKA_ISSUER);
