@@ -41,10 +41,11 @@
 #include "debug.h"
 #include "dict.h"
 #include "hash.h"
-#include "library.h"
+#include "message.h"
 #include "module.h"
 #include "oid.h"
 #include "parser.h"
+#include "path.h"
 #include "pem.h"
 #include "pkcs11x.h"
 #include "persist.h"
@@ -263,13 +264,14 @@ parse_der_x509_certificate (p11_parser *parser,
                             const unsigned char *data,
                             size_t length)
 {
+	char message[ASN1_MAX_ERROR_DESCRIPTION_SIZE];
 	CK_BYTE idv[ID_LENGTH];
 	CK_ATTRIBUTE id = { CKA_ID, idv, sizeof (idv) };
 	CK_ATTRIBUTE *attrs;
 	CK_ATTRIBUTE *value;
 	node_asn *cert;
 
-	cert = p11_asn1_decode (parser->asn1_defs, "PKIX1.Certificate", data, length, NULL);
+	cert = p11_asn1_decode (parser->asn1_defs, "PKIX1.Certificate", data, length, message);
 	if (cert == NULL)
 		return P11_PARSE_UNRECOGNIZED;
 
@@ -513,7 +515,7 @@ build_openssl_extensions (p11_parser *parser,
 	 * Otherwise a 'TRUSTED CERTIFICATE' in an input directory is enough to
 	 * mark this as a trusted certificate.
 	 */
-	} else if (p11_dict_size (trust) > 0) {
+	} else if (trust && p11_dict_size (trust) > 0) {
 		trusted = CK_TRUE;
 		distrust = CK_FALSE;
 	}
@@ -556,6 +558,7 @@ parse_openssl_trusted_certificate (p11_parser *parser,
                                    const unsigned char *data,
                                    size_t length)
 {
+	char message[ASN1_MAX_ERROR_DESCRIPTION_SIZE];
 	CK_ATTRIBUTE *attrs;
 	CK_BYTE idv[ID_LENGTH];
 	CK_ATTRIBUTE id = { CKA_ID, idv, sizeof (idv) };
@@ -578,11 +581,11 @@ parse_openssl_trusted_certificate (p11_parser *parser,
 	if (cert_len <= 0)
 		return P11_PARSE_UNRECOGNIZED;
 
-	cert = p11_asn1_decode (parser->asn1_defs, "PKIX1.Certificate", data, cert_len, NULL);
+	cert = p11_asn1_decode (parser->asn1_defs, "PKIX1.Certificate", data, cert_len, message);
 	if (cert == NULL)
 		return P11_PARSE_UNRECOGNIZED;
 
-	aux = p11_asn1_decode (parser->asn1_defs, "OPENSSL.CertAux", data + cert_len, length - cert_len, NULL);
+	aux = p11_asn1_decode (parser->asn1_defs, "OPENSSL.CertAux", data + cert_len, length - cert_len, message);
 	if (aux == NULL) {
 		asn1_delete_structure (&cert);
 		return P11_PARSE_UNRECOGNIZED;
@@ -741,7 +744,7 @@ p11_parse_memory (p11_parser *parser,
 
 	return_val_if_fail (parser != NULL, P11_PARSE_FAILURE);
 
-	base = p11_basename (filename);
+	base = p11_path_base (filename);
 	parser->basename = base;
 	parser->flags = flags;
 
